@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2014 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2016 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -67,6 +67,7 @@ $Date: 2013-04-19 12:51:22 +0200 (Fr, 19. Apr 2013) $
 #include "sketch/pcbsketchwidget.h"
 #include "help/firsttimehelpdialog.h"
 #include "help/aboutbox.h"
+#include "version/partschecker.h"
 
 // dependency injection :P
 #include "referencemodel/sqlitereferencemodel.h"
@@ -86,6 +87,8 @@ $Date: 2013-04-19 12:51:22 +0200 (Fr, 19. Apr 2013) $
 #include <QNetworkAccessManager>
 #include <QNetworkRequest>
 #include <QMultiHash>
+#include <QTemporaryFile>
+#include <time.h>
 
 #ifdef LINUX_32
 #define PLATFORM_NAME "linux-32bit"
@@ -117,6 +120,7 @@ $Date: 2013-04-19 12:51:22 +0200 (Fr, 19. Apr 2013) $
 static const double LoadProgressStart = 0.085;
 static const double LoadProgressEnd = 0.6;
 
+
 ////////////////////////////////////////////////////
 
 FServer::FServer(QObject *parent)
@@ -287,6 +291,59 @@ void FServerThread::writeResponse(QTcpSocket * socket, int code, const QString &
 
 ////////////////////////////////////////////////////
 
+RegenerateDatabaseThread::RegenerateDatabaseThread(const QString & dbFileName, QDialog * progressDialog, ReferenceModel * referenceModel) {
+    m_dbFileName = dbFileName;
+    m_referenceModel = referenceModel;
+    m_progressDialog = progressDialog;
+}
+
+const QString RegenerateDatabaseThread::error() const {
+    return m_error;
+}
+
+QDialog * RegenerateDatabaseThread::progressDialog() const {
+    return m_progressDialog;
+}
+
+ReferenceModel * RegenerateDatabaseThread::referenceModel() const {
+    return m_referenceModel;
+}
+
+void RegenerateDatabaseThread::run() {
+    QTemporaryFile file("XXXXXX.db");
+    QString fileName;
+    if (file.open()) {
+        fileName = file.fileName();
+        file.close();
+    }
+    else {
+        m_error = tr("Unable to open temporary file");
+        return;
+    }
+
+    bool ok = ((FApplication *) qApp)->loadReferenceModel(fileName, true, m_referenceModel);
+    if (!ok) {
+        m_error = tr("Database failure");
+        return;
+    }
+
+    if (QFile::exists(m_dbFileName)) {
+        ok = QFile::remove(m_dbFileName);
+        if (!ok) {
+            m_error = tr("Unable to remove original db file %1").arg(m_dbFileName);
+            return;
+        }
+    }
+
+    ok = QFile::copy(fileName, m_dbFileName);
+    if (!ok) {
+        m_error = tr("Unable to copy database file %1").arg(m_dbFileName);
+        return;
+    }
+}
+
+////////////////////////////////////////////////////
+
 FApplication::FApplication( int & argc, char ** argv) : QApplication(argc, argv)
 {
     m_fServer = NULL;
@@ -337,16 +394,41 @@ bool FApplication::init() {
 
 		if (i + 1 >= m_arguments.length()) continue;
 
-		if ((m_arguments[i].compare("-f", Qt::CaseInsensitive) == 0) ||
-			(m_arguments[i].compare("-folder", Qt::CaseInsensitive) == 0)||
-			(m_arguments[i].compare("--folder", Qt::CaseInsensitive) == 0))
-		{
-			FolderUtils::setApplicationPath(m_arguments[i + 1]);
-			// delete these so we don't try to process them as files later
-			toRemove << i << i + 1;
-		}
+        if ((m_arguments[i].compare("-f", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("-folder", Qt::CaseInsensitive) == 0)||
+            (m_arguments[i].compare("--folder", Qt::CaseInsensitive) == 0))
+        {
+            FolderUtils::setApplicationPath(m_arguments[i + 1]);
+            // delete these so we don't try to process them as files later
+            toRemove << i << i + 1;
+        }
 
-		if ((m_arguments[i].compare("-geda", Qt::CaseInsensitive) == 0) ||
+        if ((m_arguments[i].compare("-pp", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("-pa", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("-parts", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("--parts", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("--partsparent", Qt::CaseInsensitive) == 0))
+        {
+            FolderUtils::setAppPartsPath(m_arguments[i + 1]);
+            // delete these so we don't try to process them as files later
+            toRemove << i << i + 1;
+        }
+
+        if ((m_arguments[i].compare("-ov", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("-ow", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("-of", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("--ov", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("--ow", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("--of", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("--overridefolder", Qt::CaseInsensitive) == 0)
+           )
+        {
+            PaletteModel::setFzpOverrideFolder(m_arguments[i + 1]);
+            // delete these so we don't try to process them as files later
+            toRemove << i << i + 1;
+        }
+
+        if ((m_arguments[i].compare("-geda", Qt::CaseInsensitive) == 0) ||
 			(m_arguments[i].compare("--geda", Qt::CaseInsensitive) == 0)) {
 			m_serviceType = GedaService;
 			m_outputFolder = m_arguments[i + 1];
@@ -473,7 +555,6 @@ bool FApplication::init() {
 	}
 
 	m_started = false;
-	m_updateDialog = NULL;
 	m_lastTopmostWindow = NULL;
 
 	connect(&m_activationTimer, SIGNAL(timeout()), this, SLOT(updateActivation()));
@@ -539,11 +620,11 @@ FApplication::~FApplication(void)
 		
 	clearModels();
 
-	if (m_updateDialog) {
-		delete m_updateDialog;
-	}
+    if (m_updateDialog) {
+        delete m_updateDialog;
+    }
 
-	FSvgRenderer::cleanup();
+    FSvgRenderer::cleanup();
 	ViewLayer::cleanup();
 	ViewLayer::cleanup();
 	ItemBase::cleanup();
@@ -697,34 +778,43 @@ void FApplication::registerFonts() {
 
 }
 
-ReferenceModel * FApplication::loadReferenceModel(const QString & databaseName, bool fullLoad) {
-	m_referenceModel = new CurrentReferenceModel();	
-	ItemBase::setReferenceModel(m_referenceModel);
-	connect(m_referenceModel, SIGNAL(loadedPart(int, int)), this, SLOT(loadedPart(int, int)));
+bool FApplication::loadReferenceModel(const QString & databaseName, bool fullLoad) {
+    m_referenceModel = new CurrentReferenceModel();
+    ItemBase::setReferenceModel(m_referenceModel);
+    connect(m_referenceModel, SIGNAL(loadedPart(int, int)), this, SLOT(loadedPart(int, int)));
+    return loadReferenceModel(databaseName, fullLoad, m_referenceModel);
+}
 
-    bool dbExists = false;
-    QDir * dir = FolderUtils::getApplicationSubFolder("parts");
-    QString dbPath;
-    if (dir) {
-        dbPath = dir->absoluteFilePath("parts.db");
-        QFileInfo info(dbPath);
-        dbExists = info.exists();
+bool FApplication::loadReferenceModel(const QString & databaseName, bool fullLoad, ReferenceModel * referenceModel)
+{
+    QDir dir = FolderUtils::getAppPartsSubFolder("");
+    QString dbPath = dir.absoluteFilePath("parts.db");
+    QFileInfo info(dbPath);
+    bool dbExists = info.exists();
+
+    QString sha;
+    if (fullLoad) {
+        // fullLoad == true means we are creating the parts database
+        // so get the sha for last commit of the parts folder and store it in the database
+        // this sha will be used to determine whether the user's parts folder can be updated from the remote repo
+        sha = PartsChecker::getSha(dir.absolutePath());
+        if (sha.isEmpty()) {
+            return false;
+        }
+
+        referenceModel->setSha(sha);
     }
 
-	bool ok = m_referenceModel->loadAll(databaseName, fullLoad, dbExists);		// loads local parts, resource parts, and any other parts in files not in the db--these part override db parts with the same moduleID
+    // loads local parts, resource parts, and any other parts in files not in the db--these part override db parts with the same moduleID
+    bool ok = referenceModel->loadAll(databaseName, fullLoad, dbExists);
     if (ok && databaseName.isEmpty()) {
-        if (dir == NULL) {
-        }
-        else {
-            QFile file(dir->absoluteFilePath("parts.db"));
-            if (file.exists()) {
-                m_referenceModel->loadFromDB(dbPath);
-            }
+        QFile file(dir.absoluteFilePath("parts.db"));
+        if (file.exists()) {
+            referenceModel->loadFromDB(dbPath);
         }
     }
-    delete dir;
 
-	return m_referenceModel;
+    return ok;
 }
 
 MainWindow * FApplication::openWindowForService(bool lockFiles, int initialTab) {
@@ -832,7 +922,7 @@ void FApplication::runGerberServiceAux()
 
 void FApplication::initService() 
 {
-	createUserDataStoreFolderStructure();
+    createUserDataStoreFolderStructures();
 	registerFonts();
 	loadReferenceModel("", false);
 }
@@ -875,31 +965,13 @@ void FApplication::runSvgServiceAux()
 
 void FApplication::runDatabaseService()
 {
-	createUserDataStoreFolderStructure();
+    createUserDataStoreFolderStructures();
 
     DebugDialog::setEnabled(true);
-    QDir * parent = FolderUtils::getApplicationSubFolder("pdb");
-    QFileInfoList dirs = parent->entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::NoSymLinks);
-    delete parent;
 
-    QStringList nameFilters;
-    nameFilters << ("*" + FritzingPartExtension);
-    foreach (QFileInfo dirInfo, dirs) {
-        if (!dirInfo.isDir()) continue;
-
-        QDir dir(dirInfo.absoluteFilePath());
-        QFileInfoList files = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
-        foreach (QFileInfo fileInfo, files) {
-            QString path = fileInfo.absoluteFilePath();
-            QString newPath = path;
-            newPath.replace("/pdb/", "/parts/");
-            QFile::rename(path, newPath);
-        }
-    }
-
-
-    QFile::remove(m_outputFolder);
-	loadReferenceModel(m_outputFolder, true);  // m_outputFolder is actually a full path ending in ".db"
+    QString partsDB = m_outputFolder;  // m_outputFolder is actually a full path ending in ".db"
+    QFile::remove(partsDB);
+    loadReferenceModel(partsDB, true);
 }
 
 void FApplication::runGedaService() {
@@ -1105,9 +1177,13 @@ int FApplication::startup()
 
 	cleanFzzs();
 
-    createUserDataStoreFolderStructure();
+    createUserDataStoreFolderStructures();
 
-	loadReferenceModel("", false);
+    ProcessEventBlocker::processEvents();
+
+    loadReferenceModel("", false);
+
+    ProcessEventBlocker::processEvents();
 
 	QString prevVersion;
 	{
@@ -1144,15 +1220,6 @@ int FApplication::startup()
 	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.65);
 	ProcessEventBlocker::processEvents();
 
-	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.825);
-	ProcessEventBlocker::processEvents();
-
-	m_updateDialog = new UpdateDialog();
-	connect(m_updateDialog, SIGNAL(enableAgainSignal(bool)), this, SLOT(enableCheckUpdates(bool)));
-	checkForUpdates(false);
-
-	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.875);
-
 	DebugDialog::debug("load something");
 	loadSomething(prevVersion);
 	m_started = true;
@@ -1160,7 +1227,15 @@ int FApplication::startup()
 	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.99);
 	ProcessEventBlocker::processEvents();
 
+    splash.hide();
 	m_splash = NULL;
+
+    m_updateDialog = new UpdateDialog();
+    m_updateDialog->setRepoPath(FolderUtils::getAppPartsSubFolderPath(""), m_referenceModel->sha());
+    connect(m_updateDialog, SIGNAL(enableAgainSignal(bool)), this, SLOT(enableCheckUpdates(bool)));
+    connect(m_updateDialog, SIGNAL(installNewParts()), this, SLOT(installNewParts()));
+    checkForUpdates(false);
+
 	return 0;
 }
 
@@ -1293,20 +1368,18 @@ void FApplication::updatePrefs(PrefsDialog & prefsDialog)
 }
 
 void FApplication::initSplash(FSplashScreen & splash) {
-	QPixmap logo(":/resources/images/splash/fhp_logo_small.png");
 	QPixmap progress(":/resources/images/splash/splash_progressbar.png");
 
 	m_progressIndex = splash.showPixmap(progress, "progress");
 	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0);
 
 	// put this above the progress indicator
-	splash.showPixmap(logo, "fhpLogo");
 
 	QString msg1 = QObject::tr("<font face='Lucida Grande, Tahoma, Sans Serif' size='2' color='#eaf4ed'>"
-							   "&#169; 2007-%1 Fachhochschule Potsdam"
+                               "&#169; 2007-%1 Fritzing"
 							   "</font>")
-							.arg(Version::year());
-	splash.showMessage(msg1, "fhpText", Qt::AlignLeft | Qt::AlignTop);
+                            .arg(Version::year());
+    splash.showMessage(msg1, "fritzingText", Qt::AlignLeft | Qt::AlignTop);
 
 	QString msg2 = QObject::tr("<font face='Lucida Grande, Tahoma, Sans Serif' size='2' color='#eaf4ed'>"
 							   "Version %1.%2.%3 (%4%5) %6"
@@ -1317,7 +1390,14 @@ void FApplication::initSplash(FSplashScreen & splash) {
 						.arg(Version::modifier())
 						.arg(Version::shortDate())
 						.arg(m_buildType);
-	splash.showMessage(msg2, "versionText", Qt::AlignRight | Qt::AlignTop);
+    splash.showMessage(msg2, "versionText", Qt::AlignRight | Qt::AlignTop);
+
+#ifdef Q_OS_MAC
+            // remove the splash screen flag on OS-X as workaround for the reported bug
+            // https://bugreports.qt.io/browse/QTBUG-49576
+            splash.setWindowFlags(splash.windowFlags() & (~Qt::SplashScreen));
+#endif
+
     splash.show();
 }
 
@@ -1334,9 +1414,7 @@ void FApplication::checkForUpdates() {
 
 void FApplication::checkForUpdates(bool atUserRequest)
 {
-	if (atUserRequest) {
-		enableCheckUpdates(false);
-	}
+    enableCheckUpdates(false);
 
 	VersionChecker * versionChecker = new VersionChecker();
 
@@ -1362,7 +1440,7 @@ void FApplication::checkForUpdates(bool atUserRequest)
 	m_updateDialog->setVersionChecker(versionChecker);
 
 	if (atUserRequest) {
-		m_updateDialog->show();
+        m_updateDialog->exec();
 	}
 }
 
@@ -1378,23 +1456,11 @@ void FApplication::enableCheckUpdates(bool enabled)
 	//DebugDialog::debug("after enable check updates");
 }
 
-
-void FApplication::createUserDataStoreFolderStructure() {
-	// make sure that the folder structure for parts and bins, exists
-	QString userDataStorePath = FolderUtils::getUserDataStorePath();
-	QDir dataStore(userDataStorePath);
-	QStringList dataFolders = FolderUtils::getUserDataStoreFolders();
-	foreach(QString folder, dataFolders) {
-		if(!QFileInfo(dataStore.absolutePath()+folder).exists()) {
-			QString folderaux = folder.startsWith("/")? folder.remove(0,1): folder;
-			dataStore.mkpath(folder);
-		}
-	}
-
+void FApplication::createUserDataStoreFolderStructures() {
+    FolderUtils::createUserDataStoreFolders();
     FolderUtils::copyBin(BinManager::MyPartsBinLocation, BinManager::MyPartsBinTemplateLocation);
     FolderUtils::copyBin(BinManager::SearchBinLocation, BinManager::SearchBinTemplateLocation);
 	PartFactory::initFolder();
-
 }
 
 
@@ -1559,15 +1625,17 @@ bool FApplication::notify(QObject *receiver, QEvent *e)
 }
 
 void FApplication::loadSomething(const QString & prevVersion) {
-    // At this point we're trying to determine what sketches to load which are from one of the following sources:
+    // At this point we're trying to determine what sketches to load from one of the following sources:
     // Only one of these sources will actually provide sketches to load and they're listed in order of priority:
 
     //		We found sketch backups to recover
-	//		there's a previous version (open an empty sketch)
-	//		files were double-clicked
-    //		The last opened sketch
+    //		Files were double-clicked
+    //		The last opened sketch (obsolete)
     //		A new blank sketch
 
+
+    Q_UNUSED(prevVersion);
+
 	initFilesToLoad();   // sets up m_filesToLoad from the command line on PC and Linux; mac uses a FileOpen event instead
 
 	initBackups();
@@ -1575,14 +1643,8 @@ void FApplication::loadSomething(const QString & prevVersion) {
 	DebugDialog::debug("checking for backups");
     QList<MainWindow*> sketchesToLoad = recoverBackups();
 
-	bool loadPrevious = false;
-	if (sketchesToLoad.isEmpty()) {
-		loadPrevious = !prevVersion.isEmpty() && Version::greaterThan(prevVersion, Version::FirstVersionWithDetachedUserData);
-	}
 
-	DebugDialog::debug(QString("load previous %1").arg(loadPrevious));
-
-	if (!loadPrevious && sketchesToLoad.isEmpty()) {
+    if (sketchesToLoad.isEmpty()) {
 		// Check for double-clicked files to load
 		DebugDialog::debug(QString("check files to load %1").arg(m_filesToLoad.count()));
 
@@ -1599,11 +1661,14 @@ void FApplication::loadSomething(const QString & prevVersion) {
         }
 	}
 
+    ProcessEventBlocker::processEvents();
+
     // Find any previously open sketches to reload
-    if (!loadPrevious && sketchesToLoad.isEmpty()) {
-		DebugDialog::debug(QString("load last open"));
+    if (sketchesToLoad.isEmpty()) {
         // new logic here, we no longer open the most recent sketch, since it can be reached in one click from the welcome page
-		//sketchesToLoad = loadLastOpenSketch();
+
+        // DebugDialog::debug(QString("load last open"));
+        //sketchesToLoad = loadLastOpenSketch();
 	}
 
 	MainWindow * newBlankSketch = NULL;
@@ -1617,6 +1682,7 @@ void FApplication::loadSomething(const QString & prevVersion) {
 		}
 	}
 
+    ProcessEventBlocker::processEvents();
 	DebugDialog::debug(QString("finish up sketch loading"));
 
     // Finish loading the sketches and show them to the user
@@ -1625,10 +1691,7 @@ void FApplication::loadSomething(const QString & prevVersion) {
 		sketch->clearFileProgressDialog();
 	}
 
-	if (loadPrevious) {
-		doLoadPrevious(newBlankSketch);
-	}
-	else if (newBlankSketch) {
+    if (newBlankSketch) {
 		newBlankSketch->hideTempPartsBin();
         // new empty sketch defaults to welcome view
         newBlankSketch->showWelcomeView();
@@ -1655,29 +1718,6 @@ QList<MainWindow *> FApplication::loadLastOpenSketch() {
 	return sketches;
 }
 
-void FApplication::doLoadPrevious(MainWindow * sketchWindow) {
-	// Here we check if files need to be imported from an earlier version.
-    // This should be done before any files are loaded as it requires a restart.
-    // As this can generate UI it should come after the splash screen has closed.
-
-    FMessageBox messageBox(sketchWindow);
-    messageBox.setWindowTitle(tr("Import files from previous version?"));
-    messageBox.setText(tr("Do you want to import parts and bins that you have created with earlier versions of Fritzing?\n"));
-    messageBox.setInformativeText(tr("\nNote: You can import them later using the \"Help\" > \"Import parts and bins "
-									"from old version...\" menu action."));
-    messageBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
-    messageBox.setDefaultButton(QMessageBox::Cancel);
-    messageBox.setIcon(QMessageBox::Question);
-    messageBox.setWindowModality(Qt::WindowModal);
-    messageBox.setButtonText(QMessageBox::Ok, tr("Import"));
-    messageBox.setButtonText(QMessageBox::Cancel, tr("Do not import now"));
-    QMessageBox::StandardButton answer = (QMessageBox::StandardButton) messageBox.exec();
-
-    if(answer == QMessageBox::Ok) {
-		 sketchWindow->importFilesFromPrevInstall();
-    }
-}
-
 QList<MainWindow *> FApplication::recoverBackups()
 {	
 	QFileInfoList backupList;
@@ -1970,7 +2010,72 @@ void FApplication::doCommand(const QString & command, const QString & params, QS
 			status = 500;
 			result = "local zip failure";
 		}
-	}
+    }
 }
 
+void FApplication::regeneratePartsDatabase() {
+    QMessageBox messageBox(NULL);
+    messageBox.setWindowTitle(tr("Regenerate parts database?"));
+    messageBox.setText(tr("Regenerating the parts database will take some minutes and you will have to restart Fritzing\n\n") +
+                        tr("Would you like to regenerate the parts database?\n")
+                        );
+    messageBox.setInformativeText("This option is a last resort in case Fritzing's is more-or-less unable to display parts. "
+                                  "You may be better off downloading the latest Fritzing release.");
+    messageBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+    messageBox.setDefaultButton(QMessageBox::Yes);
+    messageBox.setIcon(QMessageBox::Question);
+    messageBox.setWindowModality(Qt::WindowModal);
+    messageBox.setButtonText(QMessageBox::Yes, tr("Regenerate"));
+    messageBox.setButtonText(QMessageBox::No, tr("Cancel"));
+    if ((QMessageBox::StandardButton) messageBox.exec() != QMessageBox::Yes) {
+        return;
+    }
 
+    FileProgressDialog * fileProgressDialog = new FileProgressDialog(tr("Regenerating parts database..."), 0, NULL);
+    // these don't seem very accurate (i.e. when progress is at 100%, there is still a lot of work pending)
+    // so we are leaving progress indeterminate at present
+    //connect(referenceModel, SIGNAL(partsToLoad(int)), fileProgressDialog, SLOT(setMaximum(int)));
+    //connect(referenceModel, SIGNAL(loadedPart(int,int)), fileProgressDialog, SLOT(setValue(int)));
+
+    regeneratePartsDatabaseAux(fileProgressDialog);
+}
+
+void FApplication::regeneratePartsDatabaseAux(QDialog * progressDialog) {
+    ReferenceModel * referenceModel = new CurrentReferenceModel();
+    QDir dir = FolderUtils::getAppPartsSubFolder("");
+    QString dbPath = dir.absoluteFilePath("parts.db");
+    RegenerateDatabaseThread *thread = new RegenerateDatabaseThread(dbPath, progressDialog, referenceModel);
+    connect(thread, SIGNAL(finished()), this, SLOT(regenerateDatabaseFinished()));
+    FMessageBox::BlockMessages = true;
+    thread->start();
+}
+
+void FApplication::regenerateDatabaseFinished() {
+   RegenerateDatabaseThread * thread = qobject_cast<RegenerateDatabaseThread *>(sender());
+   if (thread == NULL) return;
+
+   QDialog * progressDialog = thread->progressDialog();
+   if (progressDialog == m_updateDialog) {
+       m_updateDialog->installFinished(thread->error());
+   }
+   else {
+       if (thread->error().isEmpty()) {
+            QTimer::singleShot(50, Qt::PreciseTimer, this, SLOT(quit()));
+       }
+       else {
+           thread->referenceModel()->deleteLater();
+           QMessageBox::warning(NULL, QObject::tr("Regenerate database failed"), thread->error());
+       }
+
+       if (progressDialog) {
+           thread->progressDialog()->close();
+           thread->progressDialog()->deleteLater();
+       }
+   }
+
+   thread->deleteLater();
+}
+
+void FApplication::installNewParts() {
+    regeneratePartsDatabaseAux(m_updateDialog);
+}
