@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2014 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2016 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -223,16 +223,15 @@ QString PartFactory::getSvgFilename(ModelPart * modelPart, const QString & baseN
 {
 	QStringList tempPaths;
 	QString postfix = "/"+ SvgFilesDir +"/%1/"+ baseName;
-    QString userStore = FolderUtils::getUserDataStorePath("parts")+postfix;
+    QString userStore = FolderUtils::getUserPartsPath()+postfix;
     QString pfPath = PartFactory::folderPath() + postfix;
 	if(!modelPart->path().isEmpty()) {
         QString path = modelPart->path();
-        path.replace("/pdb/", "/parts/");
 		QDir dir(path);			// is a path to a filename
 		dir.cdUp();									// lop off the filename
 		dir.cdUp();									// parts root
 		tempPaths << dir.absolutePath() + postfix;
-		tempPaths << FolderUtils::getApplicationSubFolderPath("parts")+postfix;    // some svgs may still be in the fritzing parts folder, though the other svgs are in the user folder
+        tempPaths << FolderUtils::getAppPartsSubFolderPath("")+postfix;    // some svgs may still be in the fritzing parts folder, though the other svgs are in the user folder
         if (tempPaths.at(0).compare(userStore) != 0) {
             tempPaths << userStore;
         }
@@ -246,7 +245,7 @@ QString PartFactory::getSvgFilename(ModelPart * modelPart, const QString & baseN
     } 
 	else {
         DebugDialog::debug("modelPart with no path--this shouldn't happen");
-		tempPaths << FolderUtils::getApplicationSubFolderPath("parts")+postfix;
+        tempPaths << FolderUtils::getAppPartsSubFolderPath("")+postfix;
 		tempPaths << userStore;
 	}
 	tempPaths << ":resources/parts/svg/%1/" + baseName;
@@ -420,7 +419,7 @@ QString PartFactory::getSvgFilename(const QString & fileName)
 }
 
 bool PartFactory::svgFileExists(const QString & expectedFileName, QString & path) {
-	QString p = FolderUtils::getApplicationSubFolderPath("parts") + "/"+ SvgFilesDir + "/core/";
+    QString p = FolderUtils::getAppPartsSubFolderPath("") + "/"+ SvgFilesDir + "/core/";
 	if (QFileInfo(p + expectedFileName).exists()) {
         path = expectedFileName;
         return true;
@@ -447,7 +446,7 @@ QString PartFactory::getSvgFilenameAux(const QString & expectedFileName, GenSvg
 
 bool PartFactory::fzpFileExists(const QString & moduleID, QString & path) {
     QString expectedFileName = moduleID + FritzingPartExtension;
-	path = FolderUtils::getApplicationSubFolderPath("parts") + "/core/" + expectedFileName;
+    path = FolderUtils::getAppPartsSubFolderPath("") + "/core/" + expectedFileName;
 	if (QFileInfo(path).exists()) {
         path = expectedFileName;
         return true;
