@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2015 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2016 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -279,7 +279,7 @@ QStringList & Board::getNewImageNames() {
 void Board::fileNameEntry(const QString & filename) {
 	foreach (QString name, getImageNames()) {
 		if (filename.compare(name) == 0) {
-			QString f = FolderUtils::getApplicationSubFolderPath("parts") + "/svg/core/pcb/" + filename + ".svg";
+            QString f = FolderUtils::getAppPartsSubFolderPath("") + "/svg/core/pcb/" + filename + ".svg";
 			prepLoadImageAux(f, false);
 			return;
 		}
