@@ -1,7 +1,7 @@
 /*******************************************************************
 skw
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2015 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2016 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -854,11 +854,16 @@ void MainWindow::saveAsShareable(const QString & path, bool saveModel)
 	
 		saveParts.insert(itemBase->moduleID(), itemBase->modelPart());
 	}
-	saveBundledNonAtomicEntity(filename, FritzingBundleExtension, this, saveParts.values(), false, m_fzzFolder, saveModel, true);
+	if(alreadyHasExtension(filename, FritzingSketchExtension)) {
+		saveBundledNonAtomicEntity(filename, FritzingSketchExtension, this, saveParts.values(), false, m_fzzFolder, saveModel, true);
+	} else {
+		saveBundledNonAtomicEntity(filename, FritzingBundleExtension, this, saveParts.values(), false, m_fzzFolder, saveModel, true);
+	}
 
 }
 
 void MainWindow::saveBundledNonAtomicEntity(QString &filename, const QString &extension, Bundler *bundler, const QList<ModelPart*> &partsToSave, bool askForFilename, const QString & destFolderPath, bool saveModel, bool deleteLeftovers) {
+	bool result;
 	QStringList names;
 
 	QString fileExt;
@@ -881,7 +886,7 @@ void MainWindow::saveBundledNonAtomicEntity(QString &filename, const QString &ex
 	QString dirToRemove;
 	if (destFolderPath.isEmpty()) {
 		destFolder = QDir::temp();
-		FolderUtils::createFolderAnCdIntoIt(destFolder, TextUtils::getRandText());
+		FolderUtils::createFolderAndCdIntoIt(destFolder, TextUtils::getRandText());
 		dirToRemove = destFolder.path();
 	}
 	else {
@@ -889,13 +894,19 @@ void MainWindow::saveBundledNonAtomicEntity(QString &filename, const QString &ex
 	}
 
 	QString aux = QFileInfo(bundledFileName).fileName();
-	QString destSketchPath = // remove the last "z" from the extension
-							 destFolder.path()+"/"+aux.left(aux.size()-1);
+	QString destSketchPath;
+	if (fritzingBundleExtensions().contains(extension)) {
+		destSketchPath = // remove the last "z" from the extension
+				 destFolder.path()+"/"+aux.left(aux.size()-1);
+	} else {
+		destSketchPath = destFolder.path()+"/"+aux;
+        }
 	DebugDialog::debug("saving entity temporarily to "+destSketchPath);
 
     QStringList skipSuffixes;
 
-	if (extension.compare(FritzingBundleExtension) == 0) {
+	if (extension.compare(FritzingBundleExtension) == 0 || \
+	    extension.compare(FritzingSketchExtension) == 0 ) {
 		for (int i = 0; i < m_linkedProgramFiles.count(); i++) {
 			LinkedFile * linkedFile = m_linkedProgramFiles.at(i);
 			QFileInfo fileInfo(linkedFile->linkedFilename);
@@ -930,7 +941,13 @@ void MainWindow::saveBundledNonAtomicEntity(QString &filename, const QString &ex
 
 	ProcessEventBlocker::processEvents();
 
-	if(!FolderUtils::createZipAndSaveTo(destFolder, bundledFileName, skipSuffixes)) {
+	if (fritzingBundleExtensions().contains(extension)) {
+		result = FolderUtils::createZipAndSaveTo(destFolder, bundledFileName, skipSuffixes);
+        } else {
+		result = FolderUtils::createFZAndSaveTo(destFolder, bundledFileName, skipSuffixes);
+        }
+
+	if(!result) {
 		QMessageBox::warning(
 			this,
 			tr("Fritzing"),
@@ -1424,10 +1441,9 @@ void MainWindow::exportSpiceNetlist() {
     QString incl = ".include";
     if (output.contains(incl, Qt::CaseInsensitive)) {
         QStringList lines = output.split("\n");
-        QList<QDir *> paths;
-        paths << FolderUtils::getApplicationSubFolder("pdb");
-        paths << FolderUtils::getApplicationSubFolder("parts");
-        paths << new QDir(FolderUtils::getUserDataStorePath("parts"));
+        QList<QDir > paths;
+        paths << FolderUtils::getAppPartsSubFolder("");
+        paths << QDir(FolderUtils::getUserPartsPath());
 
         QString output2;
         foreach (QString line, lines) {
@@ -1442,9 +1458,9 @@ void MainWindow::exportSpiceNetlist() {
             QString filename = temp.trimmed();
             
             bool gotOne = false;
-            foreach (QDir * dir, paths) {
+            foreach (QDir dir, paths) {
                 foreach (QString folder, ModelPart::possibleFolders()) {
-                    QDir sub(*dir);
+                    QDir sub(dir);
                     sub.cd(folder);
                     sub.cd("spicemodels");
                     if (QFile::exists(sub.absoluteFilePath(filename))) {
@@ -1463,7 +1479,6 @@ void MainWindow::exportSpiceNetlist() {
         }
 
         output = output2;
-        foreach (QDir * dir, paths) delete dir;
     }
 
     output += ".TRAN 1ms 100ms\n";
