@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2014 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2016 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -41,14 +41,10 @@ $Date: 2013-04-29 13:10:59 +0200 (Mo, 29. Apr 2013) $
 #include "../items/moduleidnames.h"
 #include "../items/partfactory.h"
 
-bool PaletteModel::CreateTempPartsBinFile = true;
-
 static bool JustAppendAllPartsInstances = false;
 static bool FirstTime = true;
 
-QString PaletteModel::ContribPartsBinFilePath;
-
-static QString FritzingContribPath;
+QString PaletteModel::s_fzpOverrideFolder;
 
 const static QString InstanceTemplate(
         		"\t\t<instance moduleIdRef=\"%1\" path=\"%2\">\n"
@@ -94,16 +90,6 @@ PaletteModel::~PaletteModel()
 }
 
 void PaletteModel::initParts(bool dbExists) {
-	QDir * dir = FolderUtils::getApplicationSubFolder("parts");
-	if (dir == NULL) {
-	    FMessageBox::information(NULL, QObject::tr("Fritzing"),
-	                             QObject::tr("Parts folder not found.") );
-		return;
-	}
-
-	FritzingContribPath = dir->absoluteFilePath("contrib");
-	delete dir;
-
 	loadParts(dbExists);
 	if (m_root == NULL) {
 	    FMessageBox::information(NULL, QObject::tr("Fritzing"),
@@ -112,7 +98,6 @@ void PaletteModel::initParts(bool dbExists) {
 }
 
 void PaletteModel::initNames() {
-	ContribPartsBinFilePath = FolderUtils::getUserDataStorePath("bins")+"/contribParts" + FritzingBinExtension;
 }
 
 ModelPart * PaletteModel::retrieveModelPart(const QString & moduleID) {
@@ -153,29 +138,39 @@ void PaletteModel::loadParts(bool dbExists) {
 	int totalPartCount = 0;
 	emit loadedPart(0, totalPartCount);
 
-	QDir * dir1 = FolderUtils::getApplicationSubFolder("parts");
-	if (dir1 != NULL) {
-		countParts(*dir1, nameFilters, totalPartCount);
-	}
-	QDir dir2(FolderUtils::getUserDataStorePath("parts"));
-	countParts(dir2, nameFilters, totalPartCount);
-
-	QDir dir3(":/resources/parts");
-    if (m_fullLoad || !dbExists) {
-	    countParts(dir3, nameFilters, totalPartCount);
-    }
-
-	int loadingPart = 0;
-	if (dir1 != NULL) {
-		loadPartsAux(*dir1, nameFilters, loadingPart, totalPartCount);
-		delete dir1;
-	}
-
-	loadPartsAux(dir2, nameFilters, loadingPart, totalPartCount);
+    QDir dir1 = FolderUtils::getAppPartsSubFolder("");
+    QDir dir2(FolderUtils::getUserPartsPath());
+    QDir dir3(":/resources/parts");
+    QDir dir4(s_fzpOverrideFolder);
 
     if (m_fullLoad || !dbExists) {
         // otherwise these will already be in the database
-	    loadPartsAux(dir3, nameFilters, loadingPart, totalPartCount);  
+        countParts(dir1, nameFilters, totalPartCount);
+        countParts(dir3, nameFilters, totalPartCount);
+    }
+
+    if (!m_fullLoad) {
+        // don't include local parts when doing full load
+        countParts(dir2, nameFilters, totalPartCount);
+        if (!s_fzpOverrideFolder.isEmpty()) {
+            countParts(dir4, nameFilters, totalPartCount);
+        }
+    }
+
+    emit partsToLoad(totalPartCount);
+
+	int loadingPart = 0;
+    if (m_fullLoad || !dbExists) {
+        // otherwise these will already be in the database
+        loadPartsAux(dir1, nameFilters, loadingPart, totalPartCount);
+        loadPartsAux(dir3, nameFilters, loadingPart, totalPartCount);
+    }
+
+    if (!m_fullLoad) {
+        loadPartsAux(dir2, nameFilters, loadingPart, totalPartCount);
+        if (!s_fzpOverrideFolder.isEmpty()) {
+            loadPartsAux(dir4, nameFilters, loadingPart, totalPartCount);
+        }
     }
 
 	if (FirstTime) {
@@ -678,3 +673,7 @@ QList<ModelPart *> PaletteModel::allParts() {
     }
     return modelParts;
 }
+
+void PaletteModel::setFzpOverrideFolder(const QString & fzpOverrideFolder) {
+    s_fzpOverrideFolder = fzpOverrideFolder;
+}
