@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2014 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2016 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ $Date: 2013-04-29 13:10:59 +0200 (Mo, 29. Apr 2013) $
 #include "folderutils.h"
 #include "lockmanager.h"
 #include "textutils.h"
+#include "fmessagebox.h"
 #include <QDesktopServices>
 #include <QCoreApplication>
 #include <QSettings>
@@ -54,21 +55,23 @@ QString FolderUtils::m_openSaveFolder = "";
 
 FolderUtils::FolderUtils() {
 	m_openSaveFolder = ___emptyString___;
-	m_folders 
-		<< "/bins" 
-		<< "/partfactory"
-		<< "/parts/user" << "/parts/contrib"
-		<< "/parts/svg/user/icon" << "/parts/svg/user/breadboard" << "/parts/svg/user/schematic" << "/parts/svg/user/pcb"
-		<< "/parts/svg/contrib/icon" << "/parts/svg/contrib/breadboard" << "/parts/svg/contrib/schematic" << "/parts/svg/contrib/pcb"
-		<< "/backup" 
-		<< "/fzz";
+    m_userFolders
+        << "partfactory"
+        << "backup"
+        << "fzz";
+    m_documentFolders
+        << "bins"
+        << "parts/user" << "parts/contrib"
+        << "parts/svg/user/icon" << "parts/svg/user/breadboard" << "parts/svg/user/schematic" << "parts/svg/user/pcb"
+        << "parts/svg/contrib/icon" << "parts/svg/contrib/breadboard" << "parts/svg/contrib/schematic" << "parts/svg/contrib/pcb";
+
 }
 
 FolderUtils::~FolderUtils() {
 }
 
 // finds a subfolder of the application directory searching backward up the tree
-QDir * FolderUtils::getApplicationSubFolder(QString search) {
+QDir  FolderUtils::getApplicationSubFolder(QString search) {
 	if (singleton == NULL) {
 		singleton = new FolderUtils();
 	}
@@ -76,14 +79,14 @@ QDir * FolderUtils::getApplicationSubFolder(QString search) {
 	QString path = singleton->applicationDirPath();
     path += "/" + search;
 	//DebugDialog::debug(QString("path %1").arg(path) );
-    QDir* dir= new QDir(path);
-    while (!dir->exists()) {
+    QDir dir(path);
+    while (!dir.exists()) {
     	// if we're running from the debug or release folder, go up one to find things
-    	dir->cdUp();
-		dir->cdUp();
-    	if (dir->isRoot()) return NULL;   // didn't find the search folder
+        dir.cdUp();
+        dir.cdUp();
+        if (dir.isRoot()) return QDir();   // didn't find the search folder
 
-		dir->setPath(dir->absolutePath() + "/" + search);
+        dir.setPath(dir.absolutePath() + "/" + search);
    	}
 
    	return dir;
@@ -94,42 +97,93 @@ QString FolderUtils::getApplicationSubFolderPath(QString search) {
 		singleton = new FolderUtils();
 	}
 
-	QDir * dir = getApplicationSubFolder(search);
-	if (dir == NULL) return "";
-
-	QString result = dir->path();
-	delete dir;
-	return result;
+    QDir dir = getApplicationSubFolder(search);
+    return dir.path();
 }
 
-QString FolderUtils::getUserDataStorePath(QString folder) {
-	QString settingsFile = QSettings(QSettings::IniFormat,QSettings::UserScope,"Fritzing","Fritzing").fileName();
-	return QFileInfo(settingsFile).dir()
-        .absolutePath()+(folder.isEmpty()?"":QString("/")+folder);
+QString FolderUtils::getAppPartsSubFolderPath(QString search) {
+    if (singleton == NULL) {
+        singleton = new FolderUtils();
+    }
+
+    QDir dir = getAppPartsSubFolder(search);
+    return dir.path();
 }
 
-const QStringList & FolderUtils::getUserDataStoreFolders() {
-	if (singleton == NULL) {
-		singleton = new FolderUtils();
-	}
+QDir FolderUtils::getAppPartsSubFolder(QString search) {
+    if (singleton == NULL) {
+        singleton = new FolderUtils();
+    }
 
-	return singleton->userDataStoreFolders();
+    return singleton->getAppPartsSubFolder2(search);
 }
 
-bool FolderUtils::createFolderAnCdIntoIt(QDir &dir, QString newFolder) {
+QDir FolderUtils::getAppPartsSubFolder2(QString search) {
+    if (m_partsPath.isEmpty()) {
+        QDir dir = getApplicationSubFolder("fritzing-parts");
+        if (dir.exists()) {
+            m_partsPath = dir.absolutePath();
+        }
+        else {
+            QDir dir = getApplicationSubFolder("parts");
+            if (dir.exists()) {
+                m_partsPath = dir.absolutePath();
+            }
+        }
+    }
+
+
+    QString path = search.isEmpty() ? m_partsPath : m_partsPath + "/" + search;
+    //DebugDialog::debug(QString("path %1").arg(path) );
+    QDir dir(path);
+
+    return dir;
+}
+
+QString FolderUtils::getTopLevelUserDataStorePath() {
+    QString path = QSettings(QSettings::IniFormat,QSettings::UserScope,"Fritzing","Fritzing").fileName();
+    return QFileInfo(path).dir().absolutePath();
+}
+
+QString FolderUtils::getTopLevelDocumentsPath() {
+    // must add a fritzing subfolder
+    QDir dir(QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation));
+    return dir.absoluteFilePath("Fritzing");
+}
+
+QString FolderUtils::getUserBinsPath() {
+    QDir dir(getTopLevelDocumentsPath());
+    return dir.absoluteFilePath("bins");
+}
+
+QString FolderUtils::getUserPartsPath() {
+    QDir dir(getTopLevelDocumentsPath());
+    return dir.absoluteFilePath("parts");
+}
+
+bool FolderUtils::createFolderAndCdIntoIt(QDir &dir, QString newFolder) {
 	if(!dir.mkdir(newFolder)) return false;
 	if(!dir.cd(newFolder)) return false;
 
 	return true;
 }
 
-bool FolderUtils::setApplicationPath(const QString & path) 
+bool FolderUtils::setApplicationPath(const QString & path)
 {
-	if (singleton == NULL) {
-		singleton = new FolderUtils();
-	}
+    if (singleton == NULL) {
+        singleton = new FolderUtils();
+    }
 
-	return singleton->setApplicationPath2(path);
+    return singleton->setApplicationPath2(path);
+}
+
+bool FolderUtils::setAppPartsPath(const QString & path)
+{
+    if (singleton == NULL) {
+        singleton = new FolderUtils();
+    }
+
+    return singleton->setPartsPath2(path);
 }
 
 void FolderUtils::cleanup() {
@@ -162,7 +216,7 @@ const QString FolderUtils::libraryPath()
 const QString FolderUtils::applicationDirPath() {
 	if (m_appPath.isEmpty()) {
 #ifdef Q_OS_WIN
-		return QCoreApplication::applicationDirPath();
+        m_appPath = QCoreApplication::applicationDirPath();
 #else
 		// look in standard Fritzing location (applicationDirPath and parent folders) then in standard linux locations
 		QStringList candidates;
@@ -188,9 +242,9 @@ const QString FolderUtils::applicationDirPath() {
 		foreach (QString candidate, candidates) {
             //DebugDialog::debug(QString("candidate:%1").arg(candidate));
 			QDir dir(candidate);
-            if (!dir.exists("parts")) continue;
+            if (!dir.exists("translations")) continue;
 
-            if (dir.exists("bins")) {
+            if (dir.exists("help")) {
 				m_appPath = candidate;
 				return m_appPath;
 			}
@@ -206,17 +260,22 @@ const QString FolderUtils::applicationDirPath() {
 	return m_appPath;
 }
 
-bool FolderUtils::setApplicationPath2(const QString & path) 
+bool FolderUtils::setApplicationPath2(const QString & path)
 {
-	QDir dir(path);
-	if (!dir.exists()) return false;
+    QDir dir(path);
+    if (!dir.exists()) return false;
 
-	m_appPath = path;
-	return true;
+    m_appPath = path;
+    return true;
 }
 
-const QStringList & FolderUtils::userDataStoreFolders() {
-	return m_folders;
+bool FolderUtils::setPartsPath2(const QString & path)
+{
+    QDir dir(path);
+    if (!dir.exists()) return false;
+
+    m_partsPath = path;
+    return true;
 }
 
 void FolderUtils::setOpenSaveFolder(const QString& path) {
@@ -251,13 +310,8 @@ const QString FolderUtils::openSaveFolder() {
 			}
 		}
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-		DebugDialog::debug(QString("default save location: %1").arg(QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation)));
-		return QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation);
-#else
-		DebugDialog::debug(QString("default save location: %1").arg(QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation)));
-		return QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
-#endif
+        return getTopLevelDocumentsPath();
+
 	} else {
 		return m_openSaveFolder;
 	}
@@ -297,7 +351,7 @@ bool FolderUtils::isEmptyFileName(const QString &fileName, const QString &untitl
 }
 
 void FolderUtils::replicateDir(QDir srcDir, QDir targDir) {
-	// copy all files from srcDir source to tagDir
+    // copy all files from srcDir source to targDir
 	QStringList files = srcDir.entryList(QDir::AllEntries | QDir::NoDotAndDotDot);
 	for(int i=0; i < files.size(); i++) {
 		QFile tempFile(srcDir.path() + "/" +files.at(i));
@@ -340,6 +394,50 @@ void FolderUtils::rmdir(QDir & dir) {
 	dir.rmdir(dir.path());
 }
 
+
+bool FolderUtils::createFZAndSaveTo(const QDir &dirToCompress, const QString &filepath, const QStringList & skipSuffixes) {
+	DebugDialog::debug("saveASfz "+dirToCompress.path()+" into "+filepath);
+
+	QFileInfoList files=dirToCompress.entryInfoList();
+	QFile inFile;
+	
+	char c;
+
+	QString currFolderBU = QDir::currentPath();
+	QDir::setCurrent(dirToCompress.path());
+	foreach(QFileInfo file, files) {
+		if(!file.isFile()||file.fileName()==filepath) continue;
+		if (file.fileName().contains(LockManager::LockedFileName)) continue;
+
+        bool skip = false;
+        foreach (QString suffix, skipSuffixes) {
+            if (file.fileName().endsWith(suffix)) {
+                skip = true;
+                break;
+            }
+        }
+        if (skip) continue;
+
+		inFile.setFileName(file.fileName());
+
+		if(!inFile.open(QIODevice::ReadOnly)) {
+			qWarning("inFile.open(): %s", inFile.errorString().toLocal8Bit().constData());
+			return false;
+		}
+		QString destination = QFileInfo(filepath).dir().filePath(inFile.fileName());
+		if (QFileInfo(destination).exists())
+			QFile::remove(destination);
+		DebugDialog::debug("Destination " + destination);
+		inFile.copy(destination);
+
+		inFile.close();
+	}
+	QDir::setCurrent(currFolderBU);
+
+	return true;
+}
+
+
 bool FolderUtils::createZipAndSaveTo(const QDir &dirToCompress, const QString &filepath, const QStringList & skipSuffixes) {
 	DebugDialog::debug("zipping "+dirToCompress.path()+" into "+filepath);
 
@@ -417,6 +515,7 @@ bool FolderUtils::createZipAndSaveTo(const QDir &dirToCompress, const QString &f
 }
 
 
+
 bool FolderUtils::unzipTo(const QString &filepath, const QString &dirToDecompress, QString & error) {
     static QChar badCharacters[] = { '\\', '/', ':', '*', '?', '"', '<', '>', '|' };
     static QChar underscore('_');
@@ -522,6 +621,7 @@ bool FolderUtils::unzipTo(const QString &filepath, const QString &dirToDecompres
 	return true;
 }
 
+
 void FolderUtils::collectFiles(const QDir & parent, QStringList & filters, QStringList & files, bool recursive)
 {
 	QFileInfoList fileInfoList = parent.entryInfoList(filters, QDir::Files | QDir::Hidden | QDir::NoSymLinks);
@@ -540,8 +640,6 @@ void FolderUtils::collectFiles(const QDir & parent, QStringList & filters, QStri
     }
 }
 
-
-
 void FolderUtils::makePartFolderHierarchy(const QString & prefixFolder, const QString & destFolder) {
 	QDir dir(prefixFolder);
 
@@ -624,3 +722,61 @@ void FolderUtils::showInFolder(const QString & path)
     QDesktopServices::openUrl( QUrl::fromLocalFile( QFileInfo(path).absolutePath() ) );   
 #endif
 }
+
+void FolderUtils::createUserDataStoreFolders() {
+    // make sure that the folder structure for parts and bins, exists
+
+    if (singleton == NULL) {
+        singleton = new FolderUtils();
+    }
+
+    QDir userDataStore(getTopLevelUserDataStorePath());
+    foreach(QString folder, singleton->m_userFolders) {
+        QString path = userDataStore.absoluteFilePath(folder);
+        if(!QFileInfo(path).exists()) {
+            userDataStore.mkpath(folder);
+        }
+    }
+
+    QDir documents(getTopLevelDocumentsPath());
+    QStringList documentFolders(singleton->m_documentFolders);
+    foreach(QString folder, documentFolders) {
+        QString path = documents.absoluteFilePath(folder);
+        if(!QFileInfo(path).exists()) {
+            documents.mkpath(folder);
+        }
+    }
+
+    // in older versions of Fritzing, local parts and bins were in userDataStore
+    QList<QDir> toRemove;
+    QStringList folders;
+    folders << "bins" << "parts";
+    bool foundOld = false;
+    foreach(QString folder, folders ) {
+        foundOld || QFileInfo(userDataStore.absoluteFilePath(folder)).exists();
+    }
+
+    if (foundOld) {
+        // inform user about the move
+        FMessageBox::information(NULL, QCoreApplication::translate("FolderUtils", "Moving your custom parts"),
+            QCoreApplication::translate("FolderUtils", "<p>Your custom-made parts and bins are moved from the hidden "
+               "app data folder to your fritzing documents folder at <br/><br/><em>%1</em><br/><br/>"
+               "This way we hope to make it easier for you to find and edit them manually.</p>")
+               .arg(documents.absolutePath()));
+
+        // copy these into the new locations
+        foreach(QString folder, folders ) {
+            QDir source(userDataStore.absoluteFilePath(folder));
+            QDir target(documents.absoluteFilePath(folder));
+            if (source.exists()) {
+                replicateDir(source, target);
+                toRemove << source;
+            }
+        }
+
+        // now remove the obsolete locations
+        foreach (QDir dir, toRemove) {
+            rmdir(dir);
+        }
+    }
+}
