@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2014 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2016 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -419,11 +419,11 @@ bool GraphUtils::chooseRatsnestGraph(const QList<ConnectorItem *> * partConnecto
 
 	    prim_minimum_spanning_tree(g, &p[0]);
 
-	    for (std::size_t i = 0; i != p.size(); ++i) {
+        for (std::size_t i = 0; i != p.size(); ++i) {
 		    if (i == p[i]) continue;
-		    if (reverseWeights[i][p[i]] == 0) continue;
+            if (reverseWeights[(int) i][(int) p[i]] == 0) continue;
 
-		    result.insert(temp[i], temp[p[i]]);
+            result.insert(temp[(int) i], temp[(int) p[i]]);
 	    }
         retval = true;
     }
